// ==========================================================
//
// main.js
//
// Copyright (C) 2015 Interactive Research Inc.
//
// ==========================================================

// ==========================================================
// グローバル変数
// ==========================================================

var socket_id;
var target_host;
var target_port;
var cmd_index = 0;


// ==========================================================
// 関数
// ==========================================================

// ----------------------------------------------------------
// 文字列をArrayBufferに変換する（おもにchrome.socket API用）
// ----------------------------------------------------------

function str2array_buf(in_str) {

var i;

// 空の領域を用意して8bit符号なし整数の配列としてアクセス可能にする
var typed_array = new Uint8Array(in_str.length);

// 配列要素の先頭から順に、引数文字列に含まれる文字を1文字ずつ代入する
for (i=0; i<typed_array.length; i++) {
	typed_array[i] = in_str.charCodeAt(i);
}

// 生成結果をArrayBufferとして返却する
return typed_array.buffer;

}


// ----------------------------------------------------------
// ArrayBufferを文字列に変換する（おもにchrome.socket API用）
// ----------------------------------------------------------

function array_buf2str(in_array_buffer) {

var i;
var out_str = "";

// 引数のArrayBufferを8bit符号なし整数の配列としてアクセス可能にする
var typed_array = new Uint8Array(in_array_buffer);

// 配列要素を先頭から順に読み出し、用意した文字列変数に連結する
for (i=0; i<typed_array.length; i++) {
	out_str += String.fromCharCode(typed_array[i]);
}

// 生成結果を文字列として返却する
return out_str;
}


// ----------------------------------------------------------
// 文字列をArrayに変換する（おもにencoding.js用）
// ----------------------------------------------------------

function str2array(in_str) {

var i;

// 空の配列を用意する
var out_array = [];

// 配列の先頭から順に、引数文字列に含まれる文字を1文字ずつ代入する
for (i=0; i<in_str.length; i++) {
	out_array[i] = in_str.charCodeAt(i);
}

return out_array;
}


// ----------------------------------------------------------
// ステータス表示
// ----------------------------------------------------------

function print_status(text) {

// ステータスの末尾に文字列を追加
document.getElementById("result").value = document.getElementById("result").value + text + "\n";

}


// ----------------------------------------------------------
// 受信履歴表示
// ----------------------------------------------------------

function print_rx_hist(text) {

// 受信履歴の末尾に文字列を追加
document.getElementById("rx_hist").value = document.getElementById("rx_hist").value + text + "\n";

}


// ----------------------------------------------------------
// 送信履歴表示
// ----------------------------------------------------------

function print_tx_hist(text) {

// 送信履歴の末尾に文字列を追加
document.getElementById("tx_hist").value = document.getElementById("tx_hist").value + text + "\n";

}


// ----------------------------------------------------------
// 接続したらコールバックされる関数
// ----------------------------------------------------------

function on_connected_callback() {

// ステータス情報を表示
print_status("** CONNECTED **");

}


// ----------------------------------------------------------
// ソケットステータスを取得したらコールバックされる関数
// ----------------------------------------------------------

function on_gotsockstat_callback(socketInfo) {

var tx_cnvtd;
var tx_code_radio_list;
var i;
var tx_code = "A";


// ソケットが接続状態なら送信する
// Timeoutなどで対向ホストから切断された場合のハンドリングを想定している

if (socketInfo.connected) {

	// コマンド文字列を準備
	var msg_buf = document.getElementById("cmd_str").value;
	
	// 改行コードを\nに統一
	msg_buf = msg_buf.replace("\r\n", "\n");
	msg_buf = msg_buf.replace("\r", "\n");
	
	// \nをセパレータにして分割を行い、結果を配列に格納する
	var msg_array = msg_buf.split("\n");
	
	// 送信対象行が送信文字列行数の範囲なら送信
	if ((msg_array.length - 1) > cmd_index) {
	
		// 送信履歴の表示
		print_tx_hist(msg_array[cmd_index]);

		// 送信対象情報の文字コードを指定したものに変換する

		// 指定されている文字コードをラジオボックスから取り出す
		tx_code_radio_list = document.getElementsByName("tx_code_radio");
		
		for (i=0; i<tx_code_radio_list.length; i++) {
			if (tx_code_radio_list[i].checked) {
				tx_code = tx_code_radio_list[i].value;
				break;
			}
		}
		
		// ラジオボックスの指定に応じて送信データの文字コードを変換する
		switch (tx_code) {
		
		case "A":
			tx_cnvtd = Encoding.convert(str2array(msg_array[cmd_index]), 'ASCII', 'UNICODE')
			break;
		case "J":
			tx_cnvtd = Encoding.convert(str2array(msg_array[cmd_index]), 'JIS', 'UNICODE')
			break;
		case "E":
			tx_cnvtd = Encoding.convert(str2array(msg_array[cmd_index]), 'EUCJP', 'UNICODE')
			break;
		case "S":
			tx_cnvtd = Encoding.convert(str2array(msg_array[cmd_index]), 'SJIS', 'UNICODE')
			break;
		case "U":
			tx_cnvtd = Encoding.convert(str2array(msg_array[cmd_index]), 'UTF8', 'UNICODE')
			break;
		
		}

		// 送信実行
		chrome.sockets.tcp.send(socket_id, 
			str2array_buf(Encoding.codeToString(tx_cnvtd) + "\r\n"), on_sent_callback);
	
		// 行カウンタをインクリメントしてボタンに反映
		cmd_index++;
		document.getElementById("send_btn").value = (cmd_index + 1) + "行目を送信";
	
	}
	
	// 全てを送信し終わったらボタン表示を変更する
	if ((msg_array.length - 1) <= cmd_index) {
		document.getElementById("send_btn").value = "すべて送信ずみ";
		document.getElementById("send_btn").disabled = true;
	}

} else {

	// ソケットが非接続状態ならステータスに通知
	print_status("<< NO CONNECTION >>");

}

}


// ----------------------------------------------------------
// コマンド文字列を送信する（ソケットステータスのチェックのみ）
// ----------------------------------------------------------

function do_send() {

// 送信に先だってソケットの状態を取得する
chrome.sockets.tcp.getInfo(socket_id, on_gotsockstat_callback);

}


// ----------------------------------------------------------
// データを送信したらコールバックされる関数
// ----------------------------------------------------------

function on_sent_callback(info) {

// ステータス情報を表示
print_status("-- SENT --");

}


// ----------------------------------------------------------
// データを受信したらコールバックされる関数
// ----------------------------------------------------------

function on_received_callback(info) {

var rx_cnvtd;
var rx_code_radio_list;
var i;
var rx_code = "A";

// 自分のソケットかどうかチェックして、違えば直ちに終了
if (info.socketId != socket_id) {
	return;
}

// ステータス情報を表示
print_status("-- RECEIVED --");

// 指定されている文字コードをラジオボックスから取り出す
rx_code_radio_list = document.getElementsByName("rx_code_radio");

for (i=0; i<rx_code_radio_list.length; i++) {
	if (rx_code_radio_list[i].checked) {
		rx_code = rx_code_radio_list[i].value;
		break;
	}
}

// ラジオボックスの指定に応じて受信データの文字コードを変換する
switch (rx_code) {

case "A":
	rx_cnvtd = Encoding.convert(str2array(array_buf2str(info.data)), 'UNICODE', 'ASCII')
	break;
case "J":
	rx_cnvtd = Encoding.convert(str2array(array_buf2str(info.data)), 'UNICODE', 'JIS')
	break;
case "E":
	rx_cnvtd = Encoding.convert(str2array(array_buf2str(info.data)), 'UNICODE', 'EUCJP')
	break;
case "S":
	rx_cnvtd = Encoding.convert(str2array(array_buf2str(info.data)), 'UNICODE', 'SJIS')
	break;
case "U":
	rx_cnvtd = Encoding.convert(str2array(array_buf2str(info.data)), 'UNICODE', 'UTF8')
	break;

}

// 受信データを受信履歴に表示する

print_rx_hist(Encoding.codeToString(rx_cnvtd));

}


// ----------------------------------------------------------
// ソケットを生成したらコールバックされる関数
// ----------------------------------------------------------

function on_created_callback(create_info) {

// ソケットidをグローバル変数に保存
socket_id = create_info.socketId;

// ホストに接続
chrome.sockets.tcp.connect(create_info.socketId,
					target_host, target_port, on_connected_callback);

// ソケットが存在している間は切断ボタンを有効化する
document.getElementById("disconn_btn").disabled = false;

}


// ----------------------------------------------------------
// ホストに接続する
// ----------------------------------------------------------

function do_connect() {

// 接続するホストのIPアドレスとポートを準備
target_host = document.getElementById("server_name").value;
target_port = parseInt(document.getElementById("port_num").value);

// ソケットを生成
chrome.sockets.tcp.create({}, on_created_callback);

// 送信対象行を0に戻してボタンに反映する
cmd_index = 0;
document.getElementById("send_btn").value = (cmd_index + 1) + "行目を送信";
document.getElementById("send_btn").disabled = false;

}


// ----------------------------------------------------------
// ソケットをクローズしたらコールバックされる関数
// ----------------------------------------------------------

function on_closed_callback() {

// ステータス情報を表示
print_status("** SOCKET CLOSED **");

// ソケットが消滅したら切断ボタンを無効化する
document.getElementById("disconn_btn").disabled = true;

}


// ----------------------------------------------------------
// 切断したらコールバックされる関数
// ----------------------------------------------------------

function on_disconnected_callback() {

// ステータス情報を表示
print_status("** DISCONNECTED **");

// ソケットをクローズする
chrome.sockets.tcp.close(socket_id, on_closed_callback);

}


// ----------------------------------------------------------
// ホストとの接続を切断する
// ----------------------------------------------------------

function do_disconnect() {

// 切断する
chrome.sockets.tcp.disconnect(socket_id, on_disconnected_callback);

}


// ----------------------------------------------------------
// 送信履歴を消去する
// ----------------------------------------------------------

function do_clr_tx_hist() {

// 当該オブジェクトに空文字列を代入
document.getElementById("tx_hist").value = "";

}


// ----------------------------------------------------------
// 受信履歴を消去する
// ----------------------------------------------------------

function do_clr_rx_hist() {

// 当該オブジェクトに空文字列を代入
document.getElementById("rx_hist").value = "";

}


// ----------------------------------------------------------
// ステータスを消去する
// ----------------------------------------------------------

function do_clr_status() {

// 当該オブジェクトに空文字列を代入
document.getElementById("result").value = "";

}


// ----------------------------------------------------------
// 各種初期化
// ----------------------------------------------------------

function do_init() {

// ボタンにイベントハンドラを設定
document.getElementById("conn_btn").addEventListener("click", do_connect);
document.getElementById("disconn_btn").addEventListener("click", do_disconnect);
document.getElementById("send_btn").addEventListener("click", do_send);
document.getElementById("clr_tx_hist_btn").addEventListener("click", do_clr_tx_hist);
document.getElementById("clr_rx_hist_btn").addEventListener("click", do_clr_rx_hist);
document.getElementById("clr_status_btn").addEventListener("click", do_clr_status);

// データ受信時にコールバックする関数を登録
chrome.sockets.tcp.onReceive.addListener(on_received_callback);

}


// ==========================================================
// メイン
// ==========================================================

// ドキュメントのロードが完了してから初期化を開始する
window.addEventListener("load", do_init);

